package adminpart;
import java.sql.*;
import java.util.*;
import java.text.SimpleDateFormat;
import javax.swing.*;
import javax.swing.table.*;
/**
 * Esta clase implementa las operaciones que se pueden hacer sobre la BD.
 *
 * @author andrux
 */
public class OperacionesBD{
   private String bd = "rhc";
   private String login = "root";
   private String passBD = "";
   private String hostname = "localhost";
   private String url="jdbc:mysql://"+hostname+"/"+bd;

   /**
    * Fija los valores por omisión:<br><br>
    * Nombre Base de datos: "rhc"<br>
    * Nombre de usuario de BD: "root"<br>
    * Password de BD: ""<br>
    * Hostname: "localhost"<br>
    */
   public OperacionesBD(){
   }

   /**
    * Establece los parametros para la conexion.
    * @param login Nombre de usuario de la Base de Datos.
    * @param passBD Contraseña del usuario de la base datos.
    * @param hostname direccion IP, o nombre del host donde aloja la Base datos.
    * @param bd Nombre de la base de datos MySQL.
    */
    public OperacionesBD(String login, String passBD, String hostname, String bd){
        this.login=login;
        this.passBD=passBD;
        this.hostname=hostname;
        this.bd=bd;
        url = "jdbc:mysql://"+hostname+"/"+bd;
     }


   /**
    * Realiza el login, comprueba que el nombre de usuario y contraseña concuerden,
    * inicializa las variables de sesión.
    * @param user Nombre de usuario
    * @param password Contraseña de usuario.
    * @return true si el login es correcto. False si login es incorrecto.
    */
   public boolean login(String user, String password){
      try
      {
         Connection conn = null;
         Class.forName("com.mysql.jdbc.Driver").newInstance();
         conn = DriverManager.getConnection(url,login,passBD);

         if (conn != null)
         {
            Statement stmt = conn.createStatement();
            ResultSet res  = stmt.executeQuery("SELECT * FROM admin WHERE user='"+user+"'" +
                    " AND password='"+password+"'");

            while(res.next()){
               
               VariablesDeSesion.nombre  = res.getString("nombre");
               VariablesDeSesion.user = res.getString("user");
               VariablesDeSesion.password= res.getString("password");
               VariablesDeSesion.intervalos= res.getInt("intervalos");
               VariablesDeSesion.mensaje=res.getString("mensaje");
               
               res.close();
               stmt.close();
               conn.close();
               return true;
               
            }

            res.close();
            stmt.close();
            conn.close();
         }
         
       }
       catch(SQLException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(ClassNotFoundException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(InstantiationException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(IllegalAccessException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
    return false;
    }

   /**
    * Realiza un update a la información de un usuario administrador
    * @param user Nombre de usuario
    * @param password Contraseña del administrador
    * @param nombre Nombre del administrador
    * @param intervalos Intervalos de tiempo en minutos para cada reserva.
    * @param mensaje Mensaje del usuario administrador
    * @return true si el update se realiza correctamente
    */
   public boolean updateAdmin(String user, String password, String nombre, int intervalos, String mensaje){
      try
      {
         Connection conn = null;
         Class.forName("com.mysql.jdbc.Driver").newInstance();
         conn = DriverManager.getConnection(url,login,passBD);

         if (conn != null)
         {
            Statement stmt = conn.createStatement();
            int res  = stmt.executeUpdate("UPDATE admin,reservas,lunes,martes,miercoles,jueves,viernes,sabado,domingo " +
                    "SET admin.user='"+user+"', admin.password='"+password+"', admin.nombre='"+nombre+"', " +
                    "admin.intervalos="+intervalos+", admin.mensaje='"+mensaje+"', reservas.user='"+user+"', lunes.user='"+user+"', " +
                    "martes.user='"+user+"', miercoles.user='"+user+"', jueves.user='"+user+"', viernes.user='"+user+"', " +
                    "sabado.user='"+user+"', domingo.user='"+user+"' WHERE admin.user='"+VariablesDeSesion.user+"'");

            stmt.close();
            conn.close();
            return true;
         }

       }
       catch(SQLException ex)
       {
          System.out.println("Operación Update Admin error: "+ex);
       }
       catch(ClassNotFoundException ex)
       {
          System.out.println("Operación Update Admin error: "+ex);
       }
       catch(InstantiationException ex)
       {
          System.out.println("Operación Update Admin error: "+ex);
       }
       catch(IllegalAccessException ex)
       {
          System.out.println("Operación Update Admin error: "+ex);
       }
    return false;
    }

   /**
    * Realiza un update a las horas de inicio y termino de consultas de un dia
    * de la semana.
    * @param dia Dia de la semana que sera actualizado. Ej: lunes, martes, miercoles...
    * @param horaInicio formato 00:00:00
    * @param horaTermino formato 00:00:00
    * @return true si es actualizado correctamente
    */
   public boolean updateDia(String dia, String horaInicio, String horaTermino){
     try
      {
         Connection conn = null;
         Class.forName("com.mysql.jdbc.Driver").newInstance();
         conn = DriverManager.getConnection(url,login,passBD);

         if (conn != null)
         {
            Statement stmt = conn.createStatement();
            int res  = stmt.executeUpdate("UPDATE "+dia+" SET "+dia+".horainicio='"+horaInicio+"', " +
                    ""+dia+".horatermino='"+horaTermino+"' WHERE "+dia+".user='"+VariablesDeSesion.user+"'");

            stmt.close();
            conn.close();
            return true;
         }

       }
       catch(SQLException ex)
       {
          System.out.println("Operación Update "+dia+" error: "+ex);
       }
       catch(ClassNotFoundException ex)
       {
          System.out.println("Operación Update "+dia+" error: "+ex);
       }
       catch(InstantiationException ex)
       {
          System.out.println("Operación Update "+dia+" error: "+ex);
       }
       catch(IllegalAccessException ex)
       {
          System.out.println("Operación Update "+dia+" error: "+ex);
       }
    return false;
   }

   /**
    * Esta función obtiene los valores de hora de inicio y hora de término
    * del día especificado.
    * @param dia Día que se consulta (lunes, martes, miercoles...)
    * @param user Nombre de usuario
    * @return Donde:<br> [0]=nombre del día <br>[1]=Hora Inicio <br>[2]=Hora Termino
    */
   public String[] showDia(String dia, String user){
      String[] resDia=new String[3];
      resDia[0]=dia;
       try
      {
         Connection conn = null;
         Class.forName("com.mysql.jdbc.Driver").newInstance();
         conn = DriverManager.getConnection(url,login,passBD);

         if (conn != null)
         {
            Statement stmt = conn.createStatement();
            ResultSet res  = stmt.executeQuery("SELECT * FROM "+dia+" WHERE user='"+user+"'");

            while(res.next()){

               resDia[1]  = res.getString("horainicio");
               resDia[2] = res.getString("horatermino");
               
               res.close();
               stmt.close();
               conn.close();
               return resDia;

            }

            res.close();
            stmt.close();
            conn.close();
         }

       }
       catch(SQLException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(ClassNotFoundException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(InstantiationException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
       catch(IllegalAccessException ex)
       {
          System.out.println("Operación Login error: "+ex);
       }
    return resDia;
   }

   /**
    * Lista las horas del día de acuerdo al intervalo entregado.
    * @param intervalos en minutos
    * @return arreglo que contiene las horas del día de acuerdo al intervalo.
    */
   public String[] listaHoras(int intervalos){
    int total=1440/intervalos;//un día tiene 1440 minutos
    int i=1;
    String[] totalIntervalos = new String[total+1];
    totalIntervalos[0]="0";//Se agrega un nulo indicando que ese día no hay horario disponible
    Calendar calendario =new java.util.GregorianCalendar(1986, Calendar.AUGUST, 17);
    do{
      java.util.Date miDate=calendario.getTime();
      SimpleDateFormat formatoHora = new SimpleDateFormat("HH:mm:ss");
      totalIntervalos[i]=formatoHora.format(miDate);
      calendario.add(Calendar.MINUTE, intervalos);
      i++;
    }while(i<total+1);
    return totalIntervalos;
   }

 /**
  * Muestra las reservas de horas de consulta ya realizadas, y las almacena en 
  * una Tabla.
  * @param user Nombre de usuario .
  * @param tabla Tabla donde se almacenarán los datos. Antes de ser pasada debe
  * haber sido previamente inicializada con un Modelo.
  * @param modelo Modelo con el que fue inicializado la tabla.
  */
   public void verReservas( String user, JTable tabla, DefaultTableModel modelo ) {
	
    //Crear Columnas
    if(modelo.getColumnCount()==0){//Si no existen se crean
    modelo.addColumn("cód");
    modelo.addColumn("Fecha");
    modelo.addColumn("Hora");
    modelo.addColumn("Nombre");
    modelo.addColumn("Rol");
    modelo.addColumn("email");
    modelo.addColumn("Motivo");
    }

    int numFilas=modelo.getRowCount();
    System.out.println("numFilas: "+numFilas);
    if(numFilas>0){
        modelo = new DefaultTableModel();
        //tabla = new JTable(modelo);
    }
    
    try {

        Connection con = null;
         Class.forName("com.mysql.jdbc.Driver").newInstance();
         con = DriverManager.getConnection(url,login,passBD);

        /*** Ejecutar sentencia SELECT para la tabla ****/
	    Statement senten = con.createStatement();
	    ResultSet rs_sel = senten.executeQuery("SELECT * FROM `reservas` WHERE " +
                "user='"+user+"' AND fecha >= CURDATE() ORDER BY fecha, hora ASC ");

        while(rs_sel.next()){
            Object [] fila = new Object[7]; // Hay 7 columnas en la tabla
            for (int i=0;i<7;i++)
                fila[i] = rs_sel.getObject(i+1);//El primer indice en rs es el 1, no el cero, por eso se suma 1.
            modelo.addRow(fila);
            }
	    senten.close();
	    rs_sel.close();
	}

      catch(SQLException ex)
       {
          System.out.println("Operación verReserva error: "+ex);
       }
       catch(ClassNotFoundException ex)
       {
          System.out.println("Operación verReserva error: "+ex);
       }
       catch(InstantiationException ex)
       {
          System.out.println("Operación verReserva error: "+ex);
       }
       catch(IllegalAccessException ex)
       {
          System.out.println("Operación verReserva error: "+ex);
       }
     
    }

   /**
    * Borra una reserva de la base de datos.
    * @param cod Código de reserva
    * @param user Nombre de Usuario
    * @return true si la operación se hizo correctamente
    */
   public boolean eliminarReserva(int cod, String user){
       
     try
      {
         Connection conn = null;
         Class.forName("com.mysql.jdbc.Driver").newInstance();
         conn = DriverManager.getConnection(url,login,passBD);

         if (conn != null)
         {
            Statement stmt = conn.createStatement();
            int res  = stmt.executeUpdate("DELETE FROM reservas WHERE cod="+cod+" " +
                    "AND user='"+user+"'");
            if(res!=0)
                System.out.println("DELETE OK");

            stmt.close();
            conn.close();
            return true;
         }

       }
       catch(SQLException ex)
       {
          System.out.println("Operación DELETE "+cod+" error: "+ex);
       }
       catch(ClassNotFoundException ex)
       {
          System.out.println("Operación DELETE "+cod+" error: "+ex);
       }
       catch(InstantiationException ex)
       {
          System.out.println("Operación DELETE "+cod+" error: "+ex);
       }
       catch(IllegalAccessException ex)
       {
          System.out.println("Operación DELETE "+cod+" error: "+ex);
       }
    return false;
   }
}


